﻿using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Article;
using Hotline.Share.Dtos.Snapshot;

namespace Hotline.Application.Snapshot;
public interface ISnapshotApplication
{
    /// <summary>
    /// 获取用户首页数据
    /// </summary>
    /// <returns></returns>
    Task<SnapshotUserInfoOutDto> GetSnapshotUserInfoAsync();

    /// <summary>
    /// 获取小程序首页数据
    /// </summary>
    /// <returns></returns>
    Task<HomePageOutDto> GetHomePageAsync();

    /// <summary>
    /// 获取行业集合
    /// </summary>
    /// <returns></returns>
    Task<IList<IndustryOutDto>> GetIndustresAsync();

    /// <summary>
    /// 获取小程序公告列表
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    Task<IReadOnlyList<BulletinOutDto>> GetBulletinsAsync(BulletinInDto dto, CancellationToken cancellationToken);
    
    /// <summary>
    /// 获取工单列表
    /// </summary>
    Task<IList<OrderOutDto>> GetSnapshotOrdersAsync(OrderInDto dto, CancellationToken cancellationToken);

    /// <summary>
    /// 获取工单详情
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    Task<OrderPublishDetailOutDto> GetSnapshotOrderDetailAsync(string id, CancellationToken cancellationToken);

    /// <summary>
    /// 统计红包金额, 每月的总金额
    /// </summary>
    /// <param name="count"></param>
    /// <returns></returns>
    Task<IReadOnlyList<RedPackDateOutDto>> GetRedPackDateAsync(RedPackDateInDto dto, CancellationToken cancellationToken);

    /// <summary>
    /// 获取当月详细红包列表
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    Task<IList<RedPackOutDto>> GetRedPacksAsync(RedPacksInDto dto, CancellationToken cancellationToken);

    /// <summary>
    /// 获取公告详情
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    Task<BulletinOutDto> GetBulletinsDetailAsync(string id);

    /// <summary>
    /// 获取行业界面基础数据
    /// </summary>
    /// <param name="id"></param>
    /// <param name="requestAborted"></param>
    /// <returns></returns>
    Task<IndustryBaseOutDto> GetIndustryBaseAsync(string id, CancellationToken requestAborted);

    /// <summary>
    /// 添加随手拍公告
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    Task<string> AddBulletinAsync(AddSnapshotBulletinInDto dto);

    /// <summary>
    /// 审核公告
    /// </summary>
    /// <param name="examineBulletinDto"></param>
    /// <returns></returns>
    Task AuditBulletinAsync(ExamineBulletinDto examineBulletinDto);

    /// <summary>
    /// 获取公开的工单集合
    /// </summary>
    /// <param name="dto"></param>
    /// <param name="requestAborted"></param>
    /// <returns></returns>
    Task<IList<OrderPublishOutDto>> GetOrderPublishAsync(OrderPublishInDto dto, CancellationToken requestAborted);

    /// <summary>
    /// 获取公开的工单详情
    /// </summary>
    /// <param name="id"></param>
    /// <param name="requestAborted"></param>
    /// <returns></returns>
    Task<OrderPublishDetailOutDto> GetOrderPublishDetailAsync(string id, CancellationToken requestAborted);

    /// <summary>
    /// 添加公开工单
    /// </summary>
    /// <param name="addSnapshotOrderPublishInDto"></param>
    /// <param name="none"></param>
    /// <returns></returns>
    Task AddOrderPublishAsync(AddSnapshotOrderPublishInDto addSnapshotOrderPublishInDto, CancellationToken none);

    /// <summary>
    /// 获取用户领取的红包总金额
    /// </summary>
    /// <param name="requestAborted"></param>
    /// <returns></returns>
    Task<string> GetRedPackReceivedTotalAsync(CancellationToken requestAborted);

    /// <summary>
    /// 获取从业人员集合
    /// </summary>
    /// <param name="dto"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    Task<IList<PractitionerItemOutDto>> GetPractitionerItemsAsync(PractitionerItemInDto dto, CancellationToken cancellationToken);

    /// <summary>
    /// 获取从业人员详情
    /// </summary>
    /// <param name="id"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    Task<PractitionerDetailOutDto> GetPractitionerDetailAsync(string id, CancellationToken cancellationToken);

    /// <summary>
    /// 批量添加从业人员
    /// </summary>
    /// <param name="dtos"></param>
    /// <returns></returns>
    Task AddPractitionerAsync(IList<AddPractitionerInDto> dtos);
}
